@@ -1,5 +1,6 @@
 {-# LANGUAGE ScopedTypeVariables #-}
 {-# LANGUAGE DeriveDataTypeable #-}
+{-# LANGUAGE CPP #-}
 {-# OPTIONS_HADDOCK show-extensions #-}
 
 -- |
@@ -21,13 +22,16 @@ module Data.DynamicState (
 import Data.Dynamic
 import Data.HashMap.Strict as M
 import Data.ConcreteTypeRep
-import Data.Monoid
-import Control.Applicative
 
 -- | An extensible record, indexed by type
 newtype DynamicState = DynamicState { unDynamicState :: M.HashMap ConcreteTypeRep Dynamic }
   deriving (Typeable)
 
+#if __GLASGOW_HASKELL__ >= 804
+instance Semigroup DynamicState where
+  (<>) = mappend
+#endif
+
 instance Monoid DynamicState where
   mappend (DynamicState a) (DynamicState b) = DynamicState (mappend a b)
   mempty = DynamicState mempty
